@@ -3,7 +3,7 @@
 //  $Id$
 //
 //  Created by Jean-Etienne LAMIAUD on Wed Apr 27 2005.
-//  Copyright (c) 2005-2020. Jean-Etienne LAMIAUD
+//  Copyright (c) 2005-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -643,14 +643,14 @@ static BOOL isWaitingConversion = NO;
                                                         @"dcraw_file_extensions"
                                                                ofType:@"plist"];
       NSData *plistData;
-      NSString *error;
+      NSError *error;
       NSPropertyListFormat format;
       NSMutableDictionary *dict;
       plistData = [NSData dataWithContentsOfFile:cfgFile];
-      dict = [NSPropertyListSerialization propertyListFromData:plistData
-                                mutabilityOption:NSPropertyListMutableContainers
+      dict = [NSPropertyListSerialization propertyListWithData:plistData
+                                                       options:NSPropertyListMutableContainers
                                                         format:&format
-                                              errorDescription:&error];
+                                                         error:&error];
       NSAssert( dict != nil, @"Failed to read RAW files configuration" );
       cfgFileTypes = [dict objectForKey:@"extensions"];
       NSAssert( cfgFileTypes != nil,
