@@ -6,7 +6,7 @@
 //  Created by Jean-Etienne LAMIAUD on Mon Jun 27 2005.
 //
 //  Copyright (c) 2004-2005. Christophe JALADY
-//  Copyright (c) 2005-2018. Jean-Etienne LAMIAUD
+//  Copyright (c) 2005-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -280,11 +280,9 @@
 
 @implementation FFmpegReader
 
-+ (void) load  // It has the added benefit to force the runtime to load the class
++ (void) load  // Force the runtime to load the class
 {
-   // Register all formats and codecs
-   av_register_all();
-}   
+}
 
 + (void) lynkeosFileTypes:(NSArray**)fileTypes
 {
@@ -344,7 +342,7 @@
 {
    unsigned int       i;
    int                ret;
-   AVCodec           *pCodec;
+   const AVCodec           *pCodec;
    AVCodecParameters *codecParams;
    u_long             arraySize;
    BOOL               validFrame;
@@ -407,11 +405,6 @@
       // Allocate a codec context for the video stream
       _pCodecCtx = avcodec_alloc_context3(pCodec);
 
-      // Inform the codec that we can handle truncated bitstreams -- i.e.,
-      // bitstreams where frame boundaries can fall in the middle of packets
-      if ( pCodec->capabilities & AV_CODEC_CAP_TRUNCATED )
-         _pCodecCtx->flags |= AV_CODEC_FLAG_TRUNCATED;
-
       // Open codec
       if (   avcodec_parameters_to_context(_pCodecCtx, codecParams) < 0
           || avcodec_open2(_pCodecCtx, pCodec, NULL) < 0)
