@@ -3,7 +3,7 @@
 //  $Id$
 //
 //  Created by Jean-Etienne LAMIAUD on Tue Aug 30 2005.
-//  Copyright (c) 2005-2020. Jean-Etienne LAMIAUD
+//  Copyright (c) 2005-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -370,8 +370,8 @@ extern NSString * const LynkeosAlignResultRef;
  */
 - (void) getImageSample:(LynkeosImageBuffer**)buffer
                  inRect:(LynkeosIntegerRect)rect
-          withTransform:(NSAffineTransformStruct)transform
-            withOffsets:(NSPoint*)offsets;
+          withTransform:(const NSAffineTransformStruct)transform
+            withOffsets:(const NSPoint*)offsets;
 
 /*!
  * @abstract Retrieves image data in a custom format
@@ -386,8 +386,8 @@ extern NSString * const LynkeosAlignResultRef;
  *   LynkeosImageBuffer
  */
 - (LynkeosImageBuffer*) getCustomImageSampleinRect:(LynkeosIntegerRect)rect
-                                         withTransform:(NSAffineTransformStruct)transform
-                                           withOffsets:(NSPoint*)offsets;
+                                     withTransform:(const NSAffineTransformStruct)transform
+                                       withOffsets:(const NSPoint*)offsets;
 
 /*!
  * @abstract Access to the Fourier transform of the image
@@ -582,16 +582,16 @@ extern NSString * const LynkeosAlignResultRef;
 - (NSEnumerator*) imageEnumerator ;
 
 /*!
- * @abstract Get an enumerator in the given direction over the list's images 
+ * @abstract Get an enumerator in the given direction over the list's images
  *   starting at the given item
  * @param item The starting item
  * @param direct The sense (direct=YES, reverse=NO)
  * @param skip Whether to skip unselected images
  * @result The list's images enumerator
  */
-- (NSEnumerator*) imageEnumeratorStartAt:(id)item 
-                                      directSense:(BOOL)direct
-                                   skipUnselected:(BOOL)skip;
+- (NSEnumerator*) imageEnumeratorStartAt:(id)item
+                             directSense:(BOOL)direct
+                          skipUnselected:(BOOL)skip;
 
 /*!
  * @abstract Get a multi pass enumerator over the list's images, in the given
@@ -601,10 +601,10 @@ extern NSString * const LynkeosAlignResultRef;
  * @param skip Whether to skip unselected images
  * @result The list's images enumerator
  */
-- (NSEnumerator <LynkeosMultiPassEnumerator> *)
-                                    multiPassImageEnumeratorStartAt:(id)item
-                                                        directSense:(BOOL)direct
-                                                     skipUnselected:(BOOL)skip;
+- (NSEnumerator <LynkeosMultiPassEnumerator> *) multiPassImageEnumeratorStartAt:(id)item
+                                                                    directSense:(BOOL)direct
+                                                                 skipUnselected:(BOOL)skip;
+
 @end
 
 #pragma mark N   Notifications
