@@ -3,7 +3,7 @@
 //  $Id$
 //
 //  Created by Jean-Etienne LAMIAUD on Sun Nov 5 2006.
-//  Copyright (c) 2006-2020. Jean-Etienne LAMIAUD
+//  Copyright (c) 2006-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -609,7 +609,8 @@ extern NSString * const LynkeosHilightedItemDidChangeNotification;
  * @param gamma Gamma correction exponent for each plane
  */
 - (void) saveImage:(LynkeosImageBuffer*)image
-         withBlack:(double*)black white:(double*)white gamma:(double*)gamma ;
+         withBlack:(double*)black white:(double*)white gamma:(double*)gamma
+          metadata:(NSDictionary*)metadata;
 
 /*!
  * @abstract Load an image, using the registered image readers
