@@ -3,7 +3,7 @@
 //  $Id$
 //
 //  Created by Jean-Etienne LAMIAUD on Sun Mar 30 2008.
-//  Copyright (c) 2008-2020. Jean-Etienne LAMIAUD
+//  Copyright (c) 2008-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -499,7 +499,8 @@ static NSString *commonPlanesNames[3];
       _stackingFactor = 0;
       _stacker = nil;
 
-      [NSBundle loadNibNamed:@"MyChromaticAlign" owner:self];
+      [[NSBundle mainBundle] loadNibNamed:@"MyChromaticAlign" owner:self topLevelObjects:&_nibObjects];
+      [_nibObjects retain];
    }
 
    return( self );
@@ -533,6 +534,7 @@ static NSString *commonPlanesNames[3];
       [_processedImage release];
    if ( _originalOffsets != NULL )
       free(_originalOffsets);
+   [_nibObjects release];
 
    [super dealloc];
 }
