@@ -3,7 +3,7 @@
 //  $Id$
 //
 //  Created by Jean-Etienne LAMIAUD on Fri Nov 28 2003.
-//  Copyright (c) 2003-2020. Jean-Etienne LAMIAUD
+//  Copyright (c) 2003-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,14 +22,19 @@
 
 #include <limits.h>
 
-#include "LynkeosFileWriter.h"
+#include "LynkeosAlertPanel.h"
 
 #include "MyGuiConstants.h"
 #include "MyImageListWindow.h"
 
+#include "LynkeosMetadata.h"
 #include "LynkeosImageBufferAdditions.h"
 #include "LynkeosGammaCorrecter.h"
+#include "LynkeosBasicAlignResult.h"
 #include "MyUserPrefsController.h"
+#include "MyImageListEnumerator.h"
+#include "MyImageStacker.h"
+#include "MyChromaticAlignerView.h"
 
 #ifdef DOUBLE_PIXELS
 #define powerof(x,y) pow(x,y)
@@ -47,6 +52,8 @@ static NSString * const K_MARGINWIDTH_KEY = @"margin width";
 static NSString * const K_PROCESSHEIGHT_KEY = @"process height";
 static NSString * const K_COLUMNSWIDTH_KEY = @"columns width";
 
+const CFStringRef LynkeosName = CFSTR("Lynkeos");
+
 static NSCursor *myWatchCursor = nil;
 
 typedef enum { K_UP, K_DOWN } enumeration_direction_t;
@@ -97,6 +104,16 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
 }
 @end
 
+typedef struct
+{
+   MyImageListEnumerator *imgList;
+   double                 black, white, gamma;
+   u_short                nPlanes;
+   LynkeosIntegerRect     cropRectangle;
+   NSAffineTransform     *transform;
+   LynkeosImageBuffer    *sample;
+} MovieExportCtrl_t;
+
 @interface MyImageListWindow(Private)
 - (void) highlightOther:(enumeration_direction_t)sense skipUnselected:(BOOL)skip;
 - (void) itemChanged:(NSNotification*)notif ;
@@ -231,7 +248,7 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
    [buttonCell setEditable: YES];
    [buttonCell setButtonType: NSSwitchButton];
    [buttonCell setAllowsMixedState: YES];
-   [buttonCell setControlSize:NSSmallControlSize];
+   [buttonCell setControlSize:NSControlSizeSmall];
    [tableColumn setDataCell:buttonCell];
    [_textView reloadData];
 
@@ -894,6 +911,7 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
 - (void) saveImage:(LynkeosImageBuffer*)image withBlack:(double*)black
              white:(double*)white
              gamma:(double*)gamma
+          metadata:(NSDictionary*)metadata
 {
    NSEnumerator *list;
    Class writerClass;
@@ -910,7 +928,7 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
    {
       if ( [writerClass canSaveDataWithPlanes:image->_nPlanes 
                                         width:image->_w height:image->_h
-                                     metaData:nil] )
+                                     metaData:metadata] )
       {
          [_currentWriters addObject:writerClass];
          [_fileWritersMenu addItemWithTitle:[writerClass writerName]];
@@ -934,7 +952,7 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
    writerClass = [_currentWriters objectAtIndex:selectedIndex];
    [_savePanel setAllowedFileTypes:[NSArray arrayWithObject:[writerClass fileExtension]]];
 
-   if ( [_savePanel runModal] == NSFileHandlingPanelOKButton )
+   if ( [_savePanel runModal] == NSModalResponseOK )
    {
       // The user gave a filename, save the image in it
       NSURL *url = [_savePanel URL];
@@ -948,10 +966,10 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
                                              planes:image->_nPlanes 
                                              width:image->_w
                                              height:image->_h
-                                             metaData:nil];
+                                             metaData:metadata];
 
       // Let the user fine tune the writer's options
-      if ( [NSApp runModalForWindow:[writer configurationPanel]] == NSOKButton )
+      if ( [NSApp runModalForWindow:[writer configurationPanel]] == NSModalResponseOK )
       {
          // Set the watch cursor
          [myWatchCursor push];
@@ -987,9 +1005,9 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
          [writer saveImageAtURL:url
                        withData:(const REAL*const*const)[copy colorPlanes]
                      blackLevel:black[nPlanes] whiteLevel:white[nPlanes]
-                     withPlanes:image->_nPlanes
-                          width:image->_w lineWidth:image->_padw
-                         height:image->_h
+                     withPlanes:copy->_nPlanes
+                          width:copy->_w lineWidth:copy->_padw
+                         height:copy->_h
                        metaData:nil];
          [copy release];
 
@@ -1004,6 +1022,152 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
    }
 }
 
+- (void) exportMovieFromList:(id<LynkeosImageList>)list withParams:(MyImageStackerParameters*)params
+                   withBlack:(double)black white:(double)white gamma:(double)gamma
+{
+   const u_short nPlanes = list.numberOfPlanes;
+
+   LynkeosIntegerRect rect = params->_cropRectangle;
+   NSAffineTransformStruct t = [params->_transform transformStruct];
+   CGFloat scale = sqrt( t.m11*t.m22 - t.m12*t.m21 );
+   NSEnumerator *pluginList;
+   Class writerClass;
+   id <LynkeosMovieFileWriter> writer;
+   NSInteger selectedIndex = -1;
+
+   // Adjust the crop rectangle without scaling
+   rect.origin.x = rect.origin.x / scale;
+   rect.origin.y = rect.origin.y / scale;
+   rect.size.width = rect.size.width / scale;
+   rect.size.height = rect.size.height / scale;
+
+   // Construct the writers list
+   _currentWriters = [NSMutableArray array];
+   if ( [_fileWritersMenu numberOfItems] != 0 )
+      [_fileWritersMenu removeAllItems];
+   for( pluginList = [[[MyPluginsController defaultPluginController] getMovieWriters] objectEnumerator];
+       (writerClass = [pluginList nextObject]) != nil; )
+   {
+      if ( [writerClass canSaveDataWithPlanes:nPlanes
+                                        width:rect.size.width
+                                       height:rect.size.height
+                                     metaData:nil] )
+      {
+         [_currentWriters addObject:writerClass];
+         [_fileWritersMenu addItemWithTitle:[writerClass writerName]];
+      }
+   }
+   
+   // Select the last used, if any ; otherwise, select the first
+   NSString *prefWriter = [[NSUserDefaults standardUserDefaults] stringForKey:K_PREFERED_MOVIE_WRITER];
+   if ( prefWriter != nil )
+      selectedIndex = [_fileWritersMenu indexOfItemWithTitle:prefWriter];
+   if ( selectedIndex == -1 )
+      selectedIndex = 0;
+   [_fileWritersMenu selectItemAtIndex:selectedIndex];
+   
+   _savePanel = [NSSavePanel savePanel];
+   [_savePanel setTitle:NSLocalizedString(@"Export sequence",
+                                          @"Export sequence panel title")];
+   [_savePanel setCanSelectHiddenExtension:YES];
+   [_savePanel setAccessoryView:_fileWritersView];
+   writerClass = [_currentWriters objectAtIndex:selectedIndex];
+   [_savePanel setAllowedFileTypes:[NSArray arrayWithObject:[writerClass fileExtension]]];
+   
+   if ( [_savePanel runModal] == NSModalResponseOK )
+   {
+      // The user gave a filename, save the image in it
+      NSURL *url = [_savePanel URL];
+
+      selectedIndex = [_fileWritersMenu indexOfSelectedItem];
+      writerClass = [_currentWriters objectAtIndex: selectedIndex];
+
+      NSMutableDictionary *mDict
+         = [NSMutableDictionary dictionaryWithDictionary:[list getMetaData]];
+      NSArray *app = [mDict objectForKey:LynkeosMD_CreatorApp()];
+      if (app == nil)
+         [mDict setObject:[NSArray arrayWithObject:(NSString*)LynkeosName] forKey:LynkeosMD_CreatorApp()];
+      else if (![app containsObject:(NSString*)LynkeosName])
+         [mDict setObject:[app arrayByAddingObject:(NSString*)LynkeosName] forKey:LynkeosMD_CreatorApp()];
+
+      // Allocate a writer instance
+      writer = (id <LynkeosMovieFileWriter>)[writerClass
+                                             writerForURL:url
+                                             planes:nPlanes
+                                             width:rect.size.width
+                                             height:rect.size.height
+                                             metaData:mDict];
+
+      // Let the user fine tune the writer's options
+      NSWindow *cfgPanel = [writer configurationPanel];
+      if ( cfgPanel == nil || [NSApp runModalForWindow:cfgPanel] == NSModalResponseOK )
+      {
+         // Set the watch cursor
+         [myWatchCursor push];
+         // And open the progress panel
+         _exportProgressIndicator.minValue = 0.0;
+         _exportProgressIndicator.maxValue = 0.0;
+         _exportProgressIndicator.doubleValue = 0.0;
+         _exportProgressIndicator.indeterminate = YES;
+         _exportProgressIndicator.usesThreadedAnimation = YES;
+
+         // And save at last
+         MovieExportCtrl_t ctrl;
+         ctrl.imgList = (MyImageListEnumerator*)[list imageEnumeratorStartAt:nil
+                                                                 directSense:YES
+                                                              skipUnselected:YES];
+         NSAssert([ctrl.imgList isKindOfClass:[MyImageListEnumerator class]],
+                  @"Wrong class for image list enumerator");
+         ctrl.cropRectangle = rect;
+         NSAffineTransform *transform
+            = [[[NSAffineTransform alloc] initWithTransform:params->_transform] autorelease];
+         [transform scaleBy:1.0/scale];
+         ctrl.transform = transform;
+         ctrl.black = black;
+         ctrl.white = white;
+         ctrl.gamma = gamma;
+         ctrl.nPlanes = nPlanes;
+         ctrl.sample = [LynkeosImageBuffer imageBufferWithNumberOfPlanes:nPlanes
+                                                                   width:rect.size.width
+                                                                  height:rect.size.height];
+         _exportProgressSession = [NSApp beginModalSessionForWindow: _exportProgressPanel];
+         [NSApp runModalSession:_exportProgressSession];
+
+         // Count the number of images in the sequence, for the progress indicator
+         u_long total = 0;
+         while (([ctrl.imgList nextObject]) != nil)
+            total += 1;
+         _exportProgressIndicator.maxValue = total;
+         [ctrl.imgList reset];
+         [_exportProgressIndicator stopAnimation:self];
+         _exportProgressIndicator.indeterminate = NO;
+         [NSApp runModalSession:_exportProgressSession];
+
+         // And save
+         [writer saveMovieAtURL:url
+                   withDelegate:self
+                         opaque:&ctrl
+                     blackLevel:black
+                     whiteLevel:white
+                     withPlanes:nPlanes
+                          width:rect.size.width
+                         height:rect.size.height
+                       metaData:mDict];
+
+         // Remember the writer's name
+         [[NSUserDefaults standardUserDefaults]
+            setObject:[writerClass writerName] forKey:K_PREFERED_MOVIE_WRITER];
+
+         // Close the progress panel
+         [NSApp endModalSession:_exportProgressSession];
+         [_exportProgressPanel close];
+
+         // Revert the cursor to its normal state when we finished saving
+         [NSCursor pop];
+      }
+   }
+}
+
 - (LynkeosImageBuffer*) loadImage
 {
    LynkeosImageBuffer *image = nil;
@@ -1013,7 +1177,7 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
    NSArray *files;
 
    [panel setAllowedFileTypes:[fileTypes allKeys]];
-   if ( [panel runModal] == NSFileHandlingPanelOKButton )
+   if ( [panel runModal] == NSModalResponseOK )
    {
       files = [panel URLs];
 
@@ -1195,7 +1359,7 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
 
    [panel setAllowedFileTypes:[MyImageListItem imageListItemFileTypes]];
    [panel setAllowsMultipleSelection:YES];
-   if ( [panel runModal] == NSFileHandlingPanelOKButton )
+   if ( [panel runModal] == NSModalResponseOK )
    {
       URLs = [panel URLs];
 
@@ -1235,11 +1399,14 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
          [(MyDocument*)[self document] addEntry: item];
       }
       else
-         NSRunAlertPanel(NSLocalizedString(@"BadFileTitle",
-                                          @"Bad format file alert panel title"),
-                         NSLocalizedString(@"BadFile",
-                                          @"Message of bad file alert message"),
-                         nil, nil, nil, [url absoluteString] );
+      {
+         [LynkeosAlertPanel runAlertWithTitle:NSLocalizedString(@"BadFileTitle",
+                                                                @"Bad format file alert panel title")
+                                        style:NSAlertStyleCritical
+                                      message:[NSString stringWithFormat:NSLocalizedString(@"BadFile",
+                                                                                           @"Message of bad file alert message"),
+                                                                         [url absoluteString]]];
+      }
    }
 
    // Revert the cursor to its normal state when we finished adding
@@ -1358,12 +1525,139 @@ typedef enum { K_UP, K_DOWN } enumeration_direction_t;
       [item getBlackLevel:&black[i] whiteLevel:&white[i] gamma:&gamma[i]
                  forPlane:i];
 
+   NSMutableDictionary *mDict
+      = [NSMutableDictionary dictionaryWithDictionary:[item getMetaData]];
+   NSArray *app = [mDict objectForKey:LynkeosMD_CreatorApp()];
+   if (app == nil)
+      [mDict setObject:[NSArray arrayWithObject:LynkeosName] forKey:LynkeosMD_CreatorApp()];
+   else if (![app containsObject:LynkeosName])
+      [mDict setObject:[app arrayByAddingObject:LynkeosName] forKey:LynkeosMD_CreatorApp()];
+
    // Save it
-   [self saveImage:img withBlack:black white:white gamma:gamma];
+   [self saveImage:img withBlack:black white:white gamma:gamma metadata:mDict];
 }
 
 - (void) exportMovie :(id)sender
 {
+   NSAssert( _dataMode == ListData, @"Wrong data mode for exporting a movie");
+   NSAssert( _currentList != nil, @"No list to export a movie");
+
+   // Check that a crop rectangle was set
+   MyImageStackerParameters *params
+      = [_currentList getProcessingParameterWithRef:myImageStackerParametersRef
+                                      forProcessing:myImageStackerRef];
+   if (params == nil
+       || params->_cropRectangle.size.width == 0 || params->_cropRectangle.size.height == 0)
+   {
+      [LynkeosAlertPanel runAlertWithTitle:NSLocalizedString(@"NoRectForExportTitle",
+                                                             @"Title for movie export crop rectangle")
+                                     style:NSAlertStyleCritical
+                                   message:NSLocalizedString(@"NoRectForExport",
+                                                             @"Text for movie export crop rectangle")];
+      return;
+   }
+
+   double black, white, gamma;
+
+   [_imageView getBlack:&black white:&white gamma:&gamma];
+   
+   // Save it
+   [self exportMovieFromList:_currentList withParams:params
+                   withBlack:black white:white gamma:gamma];
 }
 
+- (IBAction) cancelMovieExport : (id)sender
+{
+   [NSApp stopModalWithCode:NSModalResponseCancel];
+}
+
+#pragma mark LynkeosMovieFileWriterDelegate
+
+- (void) getNextFrameWithData:(const REAL * const * *)planes
+                    lineWidth:(u_short*)lineW
+                       opaque:(void*)opaque
+                     canceled:(BOOL*)canceled
+{
+   MovieExportCtrl_t *ctrl = (MovieExportCtrl_t*)opaque;
+   LynkeosProcessableImage *img = [ctrl->imgList nextObject];
+
+   *canceled = NO; // By default
+
+   if (img != nil)
+   {
+      NSPoint offsets[3] = {0.0, 0.0, 0.0};
+      LynkeosIntegerRect r = ctrl->cropRectangle;
+      u_short x, y, c;
+
+      id <LynkeosAlignResult> alignRes
+         = (id <LynkeosAlignResult>)[img getProcessingParameterWithRef: LynkeosAlignResultRef
+                                                         forProcessing: LynkeosAlignRef];
+
+      if ( alignRes != nil )
+      {
+         NSAffineTransform *transform
+            = [[[NSAffineTransform alloc] initWithTransform:[alignRes alignTransform]] autorelease];
+         NSAffineTransformStruct t;
+
+         // Take stacking transform into account, and convert to bitmap coordinate system
+         [transform appendTransform:ctrl->transform];
+         t = [transform transformStruct];
+         const CGFloat imgHeight = [img imageSize].height;
+         t.tX += t.m21*imgHeight;
+         t.tY = (1.0 - t.m22)*imgHeight - t.tY;
+         t.m12 *= -1.0;
+         t.m21 *= -1.0;
+
+         r.origin.y =  imgHeight - r.origin.y - r.size.height;
+
+         // Take the chromatic dispersion correction into account
+         MyChromaticAlignParameter *chroma
+            = [img getProcessingParameterWithRef:myChromaticAlignerOffsetsRef
+                                   forProcessing:myChromaticAlignerRef];
+
+         // Prepare the offsets, with conversion to the bitmap coordinate system
+         for( c = 0; c < ctrl->nPlanes; c++ )
+         {
+            if ( chroma != nil )
+            {
+               offsets[c].x += chroma->_offsets[c].x;
+               offsets[c].y -= chroma->_offsets[c].y;
+            }
+         }
+
+         [img getImageSample:&ctrl->sample inRect:r withTransform:t withOffsets:offsets];
+      }
+      else
+         [img getImageSample:&ctrl->sample inRect:r];
+
+      double a = 1.0/(ctrl->white - ctrl->black);
+
+      LynkeosGammaCorrecter *gammaCorrect
+         = [LynkeosGammaCorrecter getCorrecterForGamma: ctrl->gamma];
+
+      for( c = 0; c < ctrl->nPlanes; c++ )
+      {
+         for( y = 0; y < ctrl->sample->_h; y++ )
+         {
+            for( x = 0; x < ctrl->sample->_w; x++ )
+            {
+               colorValue(ctrl->sample,x,y,c) =
+               correctedValue( gammaCorrect,
+                               (colorValue(ctrl->sample,x,y,c) - ctrl->black) * a);
+            }
+         }
+      }
+      [gammaCorrect releaseCorrecter];
+
+      *planes = (const REAL * const *)[ctrl->sample colorPlanes];
+      *lineW = ctrl->sample->_padw;
+
+      _exportProgressIndicator.doubleValue += 1.0;
+      if ([NSApp runModalSession: _exportProgressSession] == NSModalResponseCancel)
+         *canceled = YES;
+   }
+   else
+      // Finished
+      *planes = NULL;
+}
 @end
