@@ -3,7 +3,7 @@
 //  $Id$
 //
 //  Created by Jean-Etienne LAMIAUD on Wed Jun 21 2007.
-//  Copyright (c) 2007-2018. Jean-Etienne LAMIAUD
+//  Copyright (c) 2007-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -315,8 +315,7 @@ static NSString * const K_STACK_NAME = @"name";
 - (void) processStarted:(NSNotification*)notif
 {
    // We are only concerned by stacking process
-   if ( [[notif userInfo] objectForKey:LynkeosUserInfoProcess]
-        == [MyImageStacker class] )
+   if ( [[notif userInfo] objectForKey:LynkeosUserInfoProcess] == [MyImageStacker class] )
    {
       // Change the button title
       [_stackButton setTitle:NSLocalizedString(@"Stop",@"Stop button")];
@@ -445,6 +444,7 @@ static NSString * const K_STACK_NAME = @"name";
               [(MyImageListItem*)[[list imageArray] objectAtIndex:0] imageSize];
       params->_transform = [[NSAffineTransform alloc] init];
       params->_monochromeStack = NO;
+      params->_normalized = NO;
       params->_stackMethod = Stacking_Standard;
       params->_stackLock = [[NSConditionLock alloc] init];
 
@@ -453,6 +453,30 @@ static NSString * const K_STACK_NAME = @"name";
                               withRef:myImageStackerParametersRef
                         forProcessing:myImageStackerRef];
    }
+   else if (mode != ImageMode)
+   {
+      if ([[list imageArray] count] == 0)
+      {
+         // No more images in a calibration category, remove the params
+         [list setProcessingParameter:nil
+                              withRef:myImageStackerParametersRef
+                        forProcessing:myImageStackerRef];
+      }
+      else
+      {
+         LynkeosIntegerSize refSize
+            = [(MyImageListItem*)[[list imageArray] objectAtIndex:0] imageSize];
+         if (params->_cropRectangle.size.width != refSize.width
+             || params->_cropRectangle.size.height != refSize.height)
+         {
+            // New calibration frames have changed size, update the params
+            params->_cropRectangle.size = refSize;
+            [list setProcessingParameter:params
+                                 withRef:myImageStackerParametersRef
+                           forProcessing:myImageStackerRef];
+         }
+      }
+   }
 
    [_cropX setIntValue:params->_cropRectangle.origin.x];
    [_cropY setIntValue:params->_cropRectangle.origin.y];
@@ -471,8 +495,12 @@ static NSString * const K_STACK_NAME = @"name";
    state = (params->_monochromeStack ? NSOnState : NSOffState );
    if ( [_monochromeCheckBox state] != state )
       [_monochromeCheckBox setState:state];
+   state = (params->_normalized ? NSOnState : NSOffState);
+   if ( [_normalizeCheckBox state] != state)
+      [_normalizeCheckBox setState:state];
    [_doubleSizeCheckBox setEnabled:(data == ListData && mode == ImageMode)];
    [_monochromeCheckBox setEnabled:(data == ListData && mode == FlatFieldMode)];
+   [_normalizeCheckBox setEnabled:(data == ListData && mode == ImageMode)];
 
    Stack_Mode_t s_mode = (params->_stackMethod != Stacking_Calibration
                           ? params->_stackMethod
@@ -523,6 +551,7 @@ static NSString * const K_STACK_NAME = @"name";
       [_cropH setEnabled:NO];
       [_doubleSizeCheckBox setEnabled:NO];
       [_monochromeCheckBox setEnabled:NO];
+      [_normalizeCheckBox setEnabled:NO];
       [_methodPopup setEnabled:NO];
       [_methodPane selectTabViewItemAtIndex:0];
       [_stackButton setEnabled:NO];
@@ -582,7 +611,8 @@ static NSString * const K_STACK_NAME = @"name";
       _imageView = nil;
       _isStacking = NO;
 
-      [NSBundle loadNibNamed:@"MyImageStacker" owner:self];
+      [[NSBundle mainBundle] loadNibNamed:@"MyImageStacker" owner:self topLevelObjects:&_nibObjects];
+      [_nibObjects retain];
    }
 
    return( self );
@@ -607,6 +637,13 @@ static NSString * const K_STACK_NAME = @"name";
    return( self );
 }
 
+- (void) dealloc
+{
+   [_nibObjects release];
+
+   [super dealloc];
+}
+
 - (NSView*) getProcessingView
 {
    return( _panel );
@@ -753,7 +790,6 @@ static NSString * const K_STACK_NAME = @"name";
       [list setProcessingParameter:params
                            withRef:myImageStackerParametersRef
                      forProcessing:myImageStackerRef];
-//      [self highlightChange:nil];
    }
 }
 
@@ -773,6 +809,20 @@ static NSString * const K_STACK_NAME = @"name";
                   forProcessing:myImageStackerRef];
 }
 
+- (IBAction) normalizeAction :(id)sender
+{
+   id <LynkeosImageList> list = [_document currentList];
+   MyImageStackerParameters *params =
+      [list getProcessingParameterWithRef:myImageStackerParametersRef
+                            forProcessing:myImageStackerRef];
+
+   params->_normalized = ([_normalizeCheckBox state] == NSOnState);
+
+   [list setProcessingParameter:params
+                        withRef:myImageStackerParametersRef
+                  forProcessing:myImageStackerRef];
+}
+
 - (IBAction) methodChange:(id)sender
 {
    id <LynkeosImageList> list = [_document currentList];
@@ -886,6 +936,7 @@ static NSString * const K_STACK_NAME = @"name";
       [_cropH setEnabled:NO];
       [_doubleSizeCheckBox setEnabled:NO];
       [_monochromeCheckBox setEnabled:NO];
+      [_normalizeCheckBox setEnabled:NO];
 
       // Freeze the selection rectangle
       LynkeosIntegerRect r = [_imageView getSelection];
