@@ -3,7 +3,7 @@
 //  $Id$
 //
 //  Created by Jean-Etienne LAMIAUD on Wed Sep 24 2003.
-//  Copyright (c) 2003-2019. Jean-Etienne LAMIAUD
+//  Copyright (c) 2003-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include <math.h>
 
+#include "geometry.h"
 #include "ProcessStackManager.h"
 #include "MyImageView.h"
 
@@ -39,48 +40,27 @@ static const double MM_LN_10 = -M_LN10;
 @end
 
 @interface MyImageView(Tile)
-- (void) applyTile:(ImageViewTile*)tile;
+- (void) applyTileAt:(CGPoint)at;
 @end
 
 @interface ImageViewTileOperation : NSOperation
 {
 @private
    NSRect _requestedReq;
-   id <LynkeosProcessableItem> _item;
    MyImageView *_view;
 }
 
-- (id) initWithRect:(NSRect)rect inView:(NSObject *)view forItem:(id <LynkeosProcessableItem>)item;
-@end
-
-@implementation ImageViewTile
-
-- (id) initWithImage:(CGImageRef)image inRect:(CGRect)rect
-{
-   if ((self = [super init]))
-   {
-      tileImage = image;
-      tileRect = rect;
-   }
-   return self;
-}
-
-- (void) dealloc
-{
-   CGImageRelease(tileImage);
-   [super dealloc];
-}
+- (id) initWithRect:(NSRect)rect inView:(NSObject *)view;
 @end
 
 @implementation ImageViewTileOperation
 
-- (id) initWithRect: (NSRect)rect inView: (MyImageView *) view forItem:(id <LynkeosProcessableItem>)item
+- (id) initWithRect: (NSRect)rect inView: (MyImageView *)view
 {
    if ((self = [super init]) != nil)
    {
       _requestedReq = rect;
       _view = view;
-      _item = item;
    }
 
    return self;
@@ -88,19 +68,8 @@ static const double MM_LN_10 = -M_LN10;
 
 - (void) main
 {
-   // Retrieve the image (converting the coordinates to bitmap reference)
-   CGRect r = _requestedReq;
-   r.origin.y = [_item imageSize].height - r.origin.y - r.size.height;
-   CGImageRef img = [_item getImageTileInRect:r];
-
    if (!self.cancelled)
-   {
-      ImageViewTile *tile = [[ImageViewTile alloc] initWithImage:img inRect:_requestedReq];
-      if ([NSThread isMainThread])
-         [_view applyTile:tile];
-      else
-         [_view performSelectorOnMainThread:@selector(applyTile:) withObject:tile waitUntilDone:NO];
-   }
+      [_view applyTileAt:_requestedReq.origin];
 }
 @end
 
@@ -126,44 +95,54 @@ static const double MM_LN_10 = -M_LN10;
 @end
 
 @implementation MyImageView(Tile)
-- (void) applyTile:(ImageViewTile*)tile
+- (void) applyTileAt:(CGPoint)at
 {
-   CGContextRef ctx = NULL;
+   if (_drawnTile == NULL)
+      return;
 
-   if (_imageLayer != NULL)
-      ctx = CGLayerGetContext(_imageLayer);
-
-   if (ctx != NULL)
+   uint8_t *tileDrawn = &_drawnTile[(int)(at.y / K_TILE_SIZE)*_tilePerLine + (int)(at.x / K_TILE_SIZE)];
+   if (__sync_bool_compare_and_swap(tileDrawn, Blank, Drawing))
    {
-      CGContextDrawImage(ctx, tile->tileRect, tile->tileImage);
+      // It is up to us to draw it
+      CGContextRef ctx = NULL;
 
-      // Compute the view rectangle wich encloses the image tile (rotated, scaled)
-      NSPoint bottomRight = {HUGE, HUGE}, topLeft = {-HUGE, -HUGE};
-      int i = 0;
+      if (_imageLayer != NULL)
+         ctx = CGLayerGetContext(_imageLayer);
 
-      for (i = 0; i < 4; i++)
+      if (ctx != NULL)
       {
-         NSPoint p = NSMakePoint(tile->tileRect.origin.x + (CGFloat)(i % 2)*tile->tileRect.size.width,
-                                 tile->tileRect.origin.y + (CGFloat)(i / 2)*tile->tileRect.size.height);
-         p = [_imageTransform transformPoint:p];
-         if (bottomRight.x > p.x)
-            bottomRight.x = p.x;
-         if (bottomRight.y > p.y)
-            bottomRight.y = p.y;
-         if (topLeft.x < p.x)
-            topLeft.x = p.x;
-         if (topLeft.y < p.y)
-            topLeft.y = p.y;
+         // Retrieve the image (converting the coordinates to bitmap reference)
+         CGRect r = {at,{K_TILE_SIZE, K_TILE_SIZE}};
+         if (r.origin.x < 0)
+         {
+            r.size.width += r.origin.x;
+            r.origin.x = 0;
+         }
+         if ((r.origin.x + r.size.width) > _itemSize.width)
+            r.size.width = _itemSize.width - r.origin.x;
+         if (r.origin.y < 0)
+         {
+            r.size.height += r.origin.y;
+            r.origin.y = 0;
+         }
+         if ((r.origin.y + r.size.height) > _itemSize.height)
+            r.size.height = _itemSize.height - r.origin.y;
+         CGRect itemRect = r;
+         itemRect.origin.y = [_item imageSize].height - r.origin.y - r.size.height;
+         CGImageRef img = [_item getImageTileInRect:itemRect];
+
+         CGContextDrawImage(ctx, r, img);
+
+         CGImageRelease(img);
       }
-      [self setNeedsDisplayInRect:NSMakeRect(bottomRight.x, bottomRight.y,
-                                             topLeft.x - bottomRight.x, topLeft.y - bottomRight.y)];
+      *tileDrawn = Available;
+   }
+   else
+   {
+      // Wait for tile availibitity, if needed
+      while (*tileDrawn != Available)
+         sched_yield();
    }
-
-   [tile release];
-
-   if (_tilesQueue.operationCount == 0)
-      // No more tile to retrieve, redraw the image surroundings if needed
-      [self setNeedsDisplay:YES];
 }
 @end
 
@@ -296,6 +275,8 @@ static const double MM_LN_10 = -M_LN10;
    _tilesQueue = [[NSOperationQueue alloc] init];
    _layerContext = NULL;
    _imageLayer = NULL;
+   _drawnTile = NULL;
+   _tilePerLine = 0;
 
    _zoom = 1.0;
 
@@ -312,6 +293,8 @@ static const double MM_LN_10 = -M_LN10;
 
    [self initCursors];
 
+   self.canDrawConcurrently = YES;
+
     return self;
 }
 
@@ -321,6 +304,12 @@ static const double MM_LN_10 = -M_LN10;
       CGContextRelease(_layerContext);
    if ( _imageLayer != NULL )
       CGLayerRelease(_imageLayer);
+   if (_drawnTile != NULL)
+   {
+      uint8_t *ary = _drawnTile;
+      _drawnTile = NULL;
+      free(ary);
+   }
    if ( _item != nil )
       [_item release];
    if ( _imageTransform != nil )
@@ -512,6 +501,15 @@ static const double MM_LN_10 = -M_LN10;
    // Cleanup before else
    _canvasRect = NSMakeRect(0.0, 0.0, 0.0, 0.0);
    [_tilesQueue cancelAllOperations];
+   [_tilesQueue waitUntilAllOperationsAreFinished];
+#warning tiles bookkeeping could be kept (but cleared) if image size does not change
+   if (_drawnTile != NULL)
+   {
+      uint8_t *ary = _drawnTile;
+      _drawnTile = NULL;
+      free(ary);
+   }
+#warning layer could be kept if canvas size does not change
    if (_imageLayer != NULL)
    {
       CGLayerRelease(_imageLayer);
@@ -528,7 +526,6 @@ static const double MM_LN_10 = -M_LN10;
    {
       const LynkeosIntegerSize s = [_item imageSize];
 
-
       if (_itemSize.width != s.width || _itemSize.height != s.height)
          _itemSize = s;
       _canvasRect = NSMakeRect(0.0, 0.0, (CGFloat)_itemSize.width, (CGFloat)_itemSize.height);
@@ -541,9 +538,9 @@ static const double MM_LN_10 = -M_LN10;
                                                (CGBitmapInfo)kCGImageAlphaPremultipliedLast);
          _imageLayer = CGLayerCreateWithContext(_layerContext,
                                                 NSMakeSize(_itemSize.width, _itemSize.height), NULL);
-//         CGContextRef layerCtx = CGLayerGetContext(_imageLayer);
-//         CGContextSetGrayFillColor(layerCtx, 0.5, 1.0);
-//         CGContextFillRect(layerCtx, _canvasRect);
+         _tilePerLine = (_itemSize.width+K_TILE_SIZE-1)/K_TILE_SIZE;
+         _drawnTile = (uint8_t*)calloc(_tilePerLine * (_itemSize.height+K_TILE_SIZE-1)/K_TILE_SIZE,
+                                       sizeof(uint8_t));
 
          // Adjust the view frame if the image size has changed
          CGFloat x, y;
@@ -573,86 +570,6 @@ static const double MM_LN_10 = -M_LN10;
                   _canvasRect.size.height = p.y - _canvasRect.origin.y;
             }
          }
-
-         ImageViewTileOperation *firstOp = nil;
-         if ([_item supportsTiling])
-         {
-            // Gather all the image tiles in a "scattered" spiral motion starting at the view center
-            NSRect vis = self.visibleRect;
-            NSPoint center = NSMakePoint(vis.origin.x + vis.size.width/2.0, vis.origin.y + vis.size.height/2.0);
-            NSAffineTransform *t = [[[NSAffineTransform alloc] initWithTransform:_imageTransform] autorelease];
-            [t invert];
-            center = [t transformPoint:center]; // View center in image coordinates
-            // Tiles are aligned with image origin, get the center tile origin
-            center = NSMakePoint(floor(center.x/K_TILE_SIZE)*K_TILE_SIZE, floor(center.y/K_TILE_SIZE)*K_TILE_SIZE);
-            const int pseudoRadius = MAX(MAX((int)(_itemSize.width - center.x)/K_TILE_SIZE,
-                                             (int)(_itemSize.height - center.y)/K_TILE_SIZE),
-                                         MAX((int)(center.x)/K_TILE_SIZE,
-                                             (int)(center.y)/K_TILE_SIZE));
-            int maxDist;
-            for (maxDist = 0; maxDist <= pseudoRadius; maxDist++)
-            {
-               int iY;
-               for (iY = 0; ; iY++)
-               {
-                  // Get the "scattered" y
-                  const int deltY = (iY + (iY % 2))/2 * (2*(iY % 2) - 1);
-                  if (abs(deltY) > maxDist)
-                     break;
-
-                  int iX;
-                  for (iX = 0; ; iX++)
-                  {
-                     // Get the "scattered" x
-                     const int deltX = (iX + (iX % 2))/2 * (2*(iX % 2) - 1);
-                     if (abs(deltX) > maxDist)
-                        break;
-
-                     if (MAX(abs(deltX), abs(deltY)) == maxDist)
-                     {
-                        NSPoint tileOrigin = NSMakePoint(center.x + deltX*K_TILE_SIZE,
-                                                         center.y + deltY*K_TILE_SIZE);
-                        if (tileOrigin.x >= 0 && tileOrigin.y >= 0 &&
-                            tileOrigin.x < _itemSize.width && tileOrigin.y < _itemSize.height)
-                        {
-                           NSRect tileRect = NSMakeRect(tileOrigin.x, tileOrigin.y,
-                                                        MIN(K_TILE_SIZE,_itemSize.width - tileOrigin.x),
-                                                        MIN(K_TILE_SIZE, _itemSize.height - tileOrigin.y));
-                           ImageViewTileOperation *op
-                              = [[[ImageViewTileOperation alloc] initWithRect: tileRect
-                                                                       inView: self
-                                                                      forItem: _item]
-                                                         autorelease];
-                           // Keep the center tile for synchronous read (see below)
-                           if (firstOp == nil)
-                              firstOp = op;
-                           else
-                           {
-                              // Give priority to the visible part of the view
-                              if (NSIntersectsRect(vis, tileRect))
-                                 op.queuePriority = NSOperationQueuePriorityHigh;
-                              else
-                                 op.queuePriority = NSOperationQueuePriorityNormal;
-                              [_tilesQueue addOperation:op];
-                           }
-                        }
-                     }
-                  }
-               }
-            }
-
-            // Force blocking on first tile, as the view often wants redraw before any tile is available
-            if (firstOp != nil)
-            {
-               [firstOp start];
-            }
-         }
-         else
-         {
-            NSRect r = NSMakeRect(0.0, 0.0, (CGFloat)_itemSize.width, (CGFloat)_itemSize.height);
-            CGImageRef img = [_item getImageTileInRect:r];
-            [self applyTile:[[ImageViewTile alloc] initWithImage:img inRect:r]];
-         }
       }
 
       validRange = [_item isProcessed]
@@ -688,7 +605,7 @@ static const double MM_LN_10 = -M_LN10;
       [_gammaText setStringValue:@""];
    }
 
-   [self applyZoom:_zoom from:nil withRedraw:NO];
+   [self applyZoom:_zoom from:nil withRedraw:YES];
 }
 
 - (void) resetBounds
@@ -707,6 +624,80 @@ static const double MM_LN_10 = -M_LN10;
 
    if ( _imageLayer != NULL )
    {
+      if ([_item supportsTiling])
+      {
+         // Update needed tiles
+         for (CGFloat y = 0; y < _itemSize.height; y += K_TILE_SIZE)
+         {
+            for (CGFloat x = 0; x < _itemSize.width; x += K_TILE_SIZE)
+            {
+               // Calculate the corners of all tiles around this point
+               CGPoint corner[3][3];
+               for (int dy = -1; dy <= 1; dy++)
+               {
+                  for (int dx = -1; dx <= 1; dx++)
+                  {
+                     corner[1+dx][1+dy] = [_imageTransform transformPoint: NSMakePoint(x + (CGFloat)dx*K_TILE_SIZE,
+                                                                                       y + (CGFloat)dy*K_TILE_SIZE)];
+                  }
+               }
+               // Iterate on these 4 tiles
+               for (int dy = -1; dy <= 0; dy++)
+               {
+                  for (int dx = -1; dx <= 0; dx++)
+                  {
+                     if ((x + dx*K_TILE_SIZE) >= 0 && (y + dy*K_TILE_SIZE) >= 0 &&
+                         ((x + dx*K_TILE_SIZE) + K_TILE_SIZE) < _itemSize.width && ((y+dy*K_TILE_SIZE) + K_TILE_SIZE) < _itemSize.height)
+                     {
+                        // Tile exists
+                        BOOL intersects = NO;
+                        for (int cy = 0; !intersects && cy <= 1; cy++)
+                        {
+                           for (int cx = 0; !intersects && cx <= 1; cx++)
+                           {
+                              if (pointIsInsideRect(corner[1+dx+cx][1+dy+cy], rect) )
+                              {
+                                 // Tile has at least one corner inside the rectangle
+                                 intersects = YES;
+                              }
+                           }
+                        }
+                        if (!intersects)
+                        {
+                           // Check if a tile edge do intersect
+                           intersects = segmentIntersectsRect(corner[1+dx][1+dy], corner[1+dx+1][1+dy], rect)
+                                        || segmentIntersectsRect(corner[1+dx+1][1+dy], corner[1+dx+1][1+dy+1], rect)
+                                        || segmentIntersectsRect(corner[1+dx+1][1+dy+1], corner[1+dx][1+dy+1], rect)
+                                        || segmentIntersectsRect(corner[1+dx][1+dy+1], corner[1+dx][1+dy], rect);
+                           
+                        }
+                        if (intersects)
+                        {
+                           // The tile with this bottom left corner is inside the draw rect
+                           [self applyTileAt:CGPointMake(x,y)];
+                        }
+                     }
+                  }
+               }
+            }
+         }
+      }
+      else if (__sync_bool_compare_and_swap(&_drawnTile[0], Blank, Drawing))
+      {
+         CGRect r = {{0.0, 0.0}, {(CGFloat)_itemSize.width, (CGFloat)_itemSize.height}};
+         CGContextRef ctx = CGLayerGetContext(_imageLayer);
+         CGImageRef img = [_item getImageTileInRect:r];
+         CGContextDrawImage(ctx, r, img);
+         CGImageRelease(img);
+         for (int j = 0; j < (_itemSize.height + K_TILE_SIZE - 1)/K_TILE_SIZE; j++)
+            for (int i = 0; i < _tilePerLine; i++)
+               _drawnTile[i+j*_tilePerLine] = Available;
+      }
+      else
+      {
+         while (_drawnTile[0] != Available)
+            sched_yield();
+      }
       NSRect r = NSMakeRect(0.0, 0.0, _itemSize.width, _itemSize.height);
       [g saveGraphicsState];
       if ( _imageTransform != nil )
@@ -769,6 +760,13 @@ static const double MM_LN_10 = -M_LN10;
    [self applyZoom:zoom from:nil withRedraw:YES];
 }
 
+- (void) getBlack:(double*)black white:(double*)white gamma:(double*)gamma
+{
+   *black = [_blackText doubleValue];
+   *white = [_whiteText doubleValue];
+   *gamma = [_gammaText doubleValue];
+}
+
 - (LynkeosIntegerRect) getSelection
 {
    if ( _currentSelectionIndex < [_selection count])
@@ -891,16 +889,20 @@ static const double MM_LN_10 = -M_LN10;
    }
    else
    {
+      if ( selRect.size.width > _canvasRect.size.width)
+         selRect.size.width  = _canvasRect.size.width;
       if ( selRect.origin.x < _canvasRect.origin.x )
          selRect.origin.x = _canvasRect.origin.x;
       else if ( selRect.origin.x + selRect.size.width
                  > _canvasRect.origin.x + _canvasRect.size.width )
-         selRect.origin.x = _canvasRect.size.width - selRect.size.width - 1;
+         selRect.origin.x = _canvasRect.size.width - selRect.size.width;
+      if ( selRect.size.height > _canvasRect.size.height)
+         selRect.size.height  = _canvasRect.size.height;
       if ( selRect.origin.y < _canvasRect.origin.y )
          selRect.origin.y = _canvasRect.origin.y;
       else if ( selRect.origin.y + selRect.size.height
                  > _canvasRect.origin.y + _canvasRect.size.height )
-         selRect.origin.y = _canvasRect.size.height - selRect.size.height - 1;
+         selRect.origin.y = _canvasRect.size.height - selRect.size.height;
    }
 
    // Check if something has changed
