@@ -3,7 +3,7 @@
 //  $Id$
 //
 //  Created by Jean-Etienne LAMIAUD on Fri Dec 7 2007.
-//  Copyright (c) 2007-2020. Jean-Etienne LAMIAUD
+//  Copyright (c) 2007-2023. Jean-Etienne LAMIAUD
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -114,7 +114,7 @@ static int compareWavelet( const void *w1, const void *w2 )
                || _params->_wavelet[i]._frequency  > M_SQRT1_2) )
          color = [NSColor redColor];
       else
-         color = [NSColor blackColor];
+         color = [NSColor textColor];
       [text setTextColor:color];
 
       NSSlider *slider = (NSSlider*)[_levelSliderMatrix cellAtRow:i column:0];
@@ -431,7 +431,8 @@ static int compareWavelet( const void *w1, const void *w2 )
       _displayFrequency = YES;
       _progressTimer = nil;
 
-      [NSBundle loadNibNamed:@"MyWavelet" owner:self];
+      [[NSBundle mainBundle] loadNibNamed:@"MyWavelet" owner:self topLevelObjects:&_nibObjects];
+      [_nibObjects retain];
    }
 
    return( self );
@@ -443,6 +444,8 @@ static int compareWavelet( const void *w1, const void *w2 )
       [_params release];
    if ( _progressTimer != nil )
       [_progressTimer invalidate];
+   [_nibObjects release];
+
    [super dealloc];
 }
 
